SystemOrganization addCategory: #'Seaside-WebSocket-Examples'!

WARequestHandler subclass: #WAWebSocketExampleBroadcaster
	instanceVariableNames: 'sockets mutex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-WebSocket-Examples'!

!WAWebSocketExampleBroadcaster class methodsFor: 'accessing' stamp: 'pmm 8/1/2010 14:38'!
description
	^ 'WebSocket example broadcaster'! !

!WAWebSocketExampleBroadcaster class methodsFor: 'initialization' stamp: 'pmm 8/1/2010 14:17'!
initialize
	WAAdmin register: self at: 'broadcast'! !

!WAWebSocketExampleBroadcaster methodsFor: 'private' stamp: 'pmm 8/1/2010 13:52'!
addSocket: aWebSocket
	mutex critical: [
		sockets add: aWebSocket ]! !

!WAWebSocketExampleBroadcaster methodsFor: 'handling' stamp: 'pmm 8/1/2010 14:00'!
handleFiltered: aRequestContext
	| webSocket |
	webSocket := aRequestContext newWebSocket.
	
	webSocket timeout: 300. "5 minutes timeout"
	
	Transcript cr; show: 'New ', webSocket printString.

	webSocket onMessage:[:data|
		self socketsDo: [ :socket | socket send: data ] ].
	
	webSocket onClose:[
		Transcript cr; show: 'Closing ', webSocket printString.
		self removeSocket: webSocket ].
	
	webSocket onError:[ :exception |
		Transcript cr; show: webSocket.
		Transcript cr; show: exception description.
		Transcript cr; show: exception signalerContext longStack ].
	self addSocket: webSocket.
	
	"We don't need to #fork here, in fact we shouldn't since it ensures
	that shutting down WebServer shuts down any WebSockets, too."
	webSocket run! !

!WAWebSocketExampleBroadcaster methodsFor: 'initialization' stamp: 'pmm 8/1/2010 13:44'!
initialize
	super initialize.
	sockets := OrderedCollection new.
	mutex := WAMutex new! !

!WAWebSocketExampleBroadcaster methodsFor: 'private' stamp: 'pmm 8/1/2010 13:52'!
removeSocket: aWebSocket
	mutex critical: [
		sockets remove: aWebSocket ifAbsent: [ ] ]! !

!WAWebSocketExampleBroadcaster methodsFor: 'private' stamp: 'pmm 8/1/2010 13:51'!
socketsDo: aOneArgumentBlock
	mutex critical: [
		sockets do: aOneArgumentBlock ]! !

WAComponent subclass: #WAWebSocketExample
	instanceVariableNames: 'sockets mutex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-WebSocket-Examples'!

!WAWebSocketExample class methodsFor: 'initialization' stamp: 'pmm 8/1/2010 14:14'!
initialize
	WAAdmin register: self asApplicationAt: 'websocket/chat'! !

!WAWebSocketExample methodsFor: 'rendering' stamp: 'pmm 8/1/2010 14:35'!
renderContentOn: html
	html paragraph
		id: 'status';
		with: 'WebSocket: Not connected'.
	html basicForm: [
		html textInput
			id: 'chat';
			placeholder: 'type and press enter to chat' ].
	html paragraph: 'To test, open two windows with Web Socket support, type a message above and press return.'.
	html unorderedList
		id: 'log'.
		
	self renderJavaScriptOn: html! !

!WAWebSocketExample methodsFor: 'rendering' stamp: 'pmm 8/1/2010 14:34'!
renderJavaScriptOn: html
	html script: '
var conn = {};
var log = document.getElementById("log");
var chat = document.getElementById("chat");
var form = chat.form;
var state = document.getElementById("status"); 
 
if (window.WebSocket === undefined) {
  state.innerHTML = "(WebSockets not supported)";
} else {
    conn = new WebSocket("ws://localhost:8080/broadcast");
    conn.onopen = function () {
      state.innerHTML = "WebSocket: Connected";
    };
 
    conn.onmessage = function (event) {
      log.innerHTML = "<li>" + event.data + "</li>" + log.innerHTML;
    };
    
    conn.onclose = function (event) {
      state.innerHTML = "WebSocket: Closed";
    };

	form.addEventListener("submit", function (event) {
		event.preventDefault();
		if (conn.readyState === 1) {
			conn.send(chat.value);      
			chat.value = "";
		}
	}, false);
};'! !

!WAWebSocketExample methodsFor: 'updating' stamp: 'pmm 8/1/2010 14:04'!
updateRoot: root
	super updateRoot: root.
	root title: 'WebSocket Example'
	! !
WAWebSocketExampleBroadcaster initialize!
WAWebSocketExample initialize!
